@@ -30,7 +30,7 @@
  * you, for example preroll, clock synchronization, state changes, activation in
  * push or pull mode, and queries. In most cases, when writing sink elements,
  * there is no need to implement class methods from #GstElement or to set
- * functions on pads, because the #GstBaseSink infrastructure is sufficient.
+ * functions on pads, because the #GstBaseSink infrastructure should be sufficient.
  *
  * There is only support in GstBaseSink for one sink pad, which should be named
  * "sink". A sink implementation (subclass of GstBaseSink) should install a pad
@@ -53,8 +53,8 @@
  * #GstBaseSink will handle the prerolling correctly. This means that it will
  * return GST_STATE_CHANGE_ASYNC from a state change to PAUSED until the first buffer
  * arrives in this element. The base class will call the GstBaseSink::preroll
- * vmethod with this preroll buffer and will then commit the state change to
- * PAUSED. 
+ * vmethod with this preroll buffer and will then commit the state change to the
+ * next asynchronously pending state.
  *
  * When the element is set to PLAYING, #GstBaseSink will synchronize on the clock
  * using the times returned from ::get_times. If this function returns
@@ -88,7 +88,7 @@
  * query will be forwarded upstream.
  *
  * The ::set_caps function will be called when the subclass should configure itself
- * to precess a specific media type.
+ * to process a specific media type.
  * 
  * The ::start and ::stop virtual methods will be called when resources should be
  * allocated. Any ::preroll, ::render  and ::set_caps function will be called
@@ -106,7 +106,16 @@
  * operations they perform in the ::render method. This is mostly usefull when
  * the ::render method performs a blocking write on a file descripter.
  *
- * Last reviewed on 2006-01-30 (0.10.3)
+ * The max-lateness property defines how the sink deals with buffers that arrive 
+ * too late in the sink. A buffer arrives too late in the sink when the presentation
+ * time (as a combination of the last segment, buffer timestamp and element
+ * base_time) is before the current time of the clock. If the frame is later than
+ * max-lateness, the sink will drop the buffer and will generate a QoS event upstream.
+ * This feature is disabled if sync is disabled, the ::get-times method does not
+ * return a valid start time or max-lateness is set to -1 (the default). 
+ * Subclasses can use gst_base_sink_set_max_lateness() to configure the max-lateness.
+ *
+ * Last reviewed on 2006-03-07 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -132,13 +141,16 @@ enum
 #define DEFAULT_CAN_ACTIVATE_PULL FALSE /* fixme: enable me */
 #define DEFAULT_CAN_ACTIVATE_PUSH TRUE
 
-#define DEFAULT_SYNC TRUE
+#define DEFAULT_PREROLL_QUEUE_LEN 	0
+#define DEFAULT_SYNC 			TRUE
+#define DEFAULT_MAX_LATENESS		-1
 
 enum
 {
   PROP_0,
   PROP_PREROLL_QUEUE_LEN,
-  PROP_SYNC
+  PROP_SYNC,
+  PROP_MAX_LATENESS
 };
 
 static GstElementClass *parent_class = NULL;
@@ -226,14 +238,21 @@ gst_base_sink_class_init (GstBaseSinkClass * klass)
    * upstream element, ie, the BUFFER_SIZE event. */
   g_object_class_install_property (G_OBJECT_CLASS (klass),
       PROP_PREROLL_QUEUE_LEN,
-      g_param_spec_uint ("preroll-queue-len", "preroll-queue-len",
-          "Number of buffers to queue during preroll", 0, G_MAXUINT, 0,
-          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+      g_param_spec_uint ("preroll-queue-len", "Preroll queue length",
+          "Number of buffers to queue during preroll", 0, G_MAXUINT,
+          DEFAULT_PREROLL_QUEUE_LEN, G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SYNC,
       g_param_spec_boolean ("sync", "Sync", "Sync on the clock", DEFAULT_SYNC,
           G_PARAM_READWRITE));
 
+  g_object_class_install_property (G_OBJECT_CLASS (klass),
+      PROP_MAX_LATENESS,
+      g_param_spec_int64 ("max-lateness", "Max Lateness",
+          "Maximum number of nanoseconds that a buffer can be late before it "
+          "is dropped (-1 unlimited)", -1, G_MAXINT64, DEFAULT_MAX_LATENESS,
+          G_PARAM_READWRITE));
+
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_base_sink_change_state);
   gstelement_class->send_event = GST_DEBUG_FUNCPTR (gst_base_sink_send_event);
@@ -342,6 +361,7 @@ gst_base_sink_init (GstBaseSink * basesink, gpointer g_class)
   basesink->pad_mode = GST_ACTIVATE_NONE;
   basesink->preroll_queue = g_queue_new ();
   basesink->abidata.ABI.clip_segment = gst_segment_new ();
+  basesink->abidata.ABI.max_lateness = -1;
 
   basesink->can_activate_push = DEFAULT_CAN_ACTIVATE_PUSH;
   basesink->can_activate_pull = DEFAULT_CAN_ACTIVATE_PULL;
@@ -364,6 +384,95 @@ gst_base_sink_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+/**
+ * gst_base_sink_set_sync:
+ * @sink: the sink
+ * @sync: the new sync value.
+ *
+ * Configures @sink to synchronize on the clock or not. When
+ * @sync is FALSE, incomming samples will be played as fast as
+ * possible. If @sync is TRUE, the timestamps of the incomming
+ * buffers will be used to schedule the exact render time of its
+ * contents.
+ *
+ * Since: 0.10.4
+ */
+void
+gst_base_sink_set_sync (GstBaseSink * sink, gboolean sync)
+{
+  GST_OBJECT_LOCK (sink);
+  sink->sync = sync;
+  GST_OBJECT_UNLOCK (sink);
+}
+
+/**
+ * gst_base_sink_get_sync:
+ * @sink: the sink
+ *
+ * Checks if @sink is currently configured to synchronize against the
+ * clock.
+ *
+ * Returns: TRUE if the sink is configured to synchronize against the clock.
+ *
+ * Since: 0.10.4
+ */
+gboolean
+gst_base_sink_get_sync (GstBaseSink * sink)
+{
+  gboolean res;
+
+  GST_OBJECT_LOCK (sink);
+  res = sink->sync;
+  GST_OBJECT_UNLOCK (sink);
+
+  return res;
+}
+
+/**
+ * gst_base_sink_set_max_lateness:
+ * @sink: the sink
+ * @max_lateness: the new max lateness value.
+ *
+ * Sets the new max lateness value to @max_lateness. This value is
+ * used to decide if a buffer should be dropped or not based on the
+ * buffer timestamp and the current clock time. A value of -1 means
+ * an unlimited time.
+ *
+ * Since: 0.10.4
+ */
+void
+gst_base_sink_set_max_lateness (GstBaseSink * sink, gint64 max_lateness)
+{
+  GST_OBJECT_LOCK (sink);
+  sink->abidata.ABI.max_lateness = max_lateness;
+  GST_OBJECT_UNLOCK (sink);
+}
+
+/**
+ * gst_base_sink_get_max_lateness:
+ * @sink: the sink
+ *
+ * Gets the max lateness value. See gst_base_sink_set_max_lateness for
+ * more details.
+ *
+ * Returns: The maximum time in nanoseconds that a buffer can be late
+ * before it is dropped and not rendered. A value of -1 means an
+ * unlimited time.
+ *
+ * Since: 0.10.4
+ */
+gint64
+gst_base_sink_get_max_lateness (GstBaseSink * sink)
+{
+  gint64 res;
+
+  GST_OBJECT_LOCK (sink);
+  res = sink->abidata.ABI.max_lateness;
+  GST_OBJECT_UNLOCK (sink);
+
+  return res;
+}
+
 static void
 gst_base_sink_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -378,9 +487,10 @@ gst_base_sink_set_property (GObject * object, guint prop_id,
       GST_PAD_PREROLL_UNLOCK (sink->sinkpad);
       break;
     case PROP_SYNC:
-      GST_OBJECT_LOCK (sink);
-      sink->sync = g_value_get_boolean (value);
-      GST_OBJECT_UNLOCK (sink);
+      gst_base_sink_set_sync (sink, g_value_get_boolean (value));
+      break;
+    case PROP_MAX_LATENESS:
+      gst_base_sink_set_max_lateness (sink, g_value_get_int64 (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -401,9 +511,10 @@ gst_base_sink_get_property (GObject * object, guint prop_id, GValue * value,
       GST_PAD_PREROLL_UNLOCK (sink->sinkpad);
       break;
     case PROP_SYNC:
-      GST_OBJECT_LOCK (sink);
-      g_value_set_boolean (value, sink->sync);
-      GST_OBJECT_UNLOCK (sink);
+      g_value_set_boolean (value, gst_base_sink_get_sync (sink));
+      break;
+    case PROP_MAX_LATENESS:
+      g_value_set_int64 (value, gst_base_sink_get_max_lateness (sink));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -411,6 +522,7 @@ gst_base_sink_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
+
 static GstCaps *
 gst_base_sink_get_caps (GstBaseSink * sink)
 {
@@ -598,7 +710,9 @@ gst_base_sink_get_sync_times (GstBaseSink * basesink, GstMiniObject * obj,
         return TRUE;
         /* other events do not need syncing */
         /* FIXME, maybe NEWSEGMENT might need synchronisation
-         * since the POSITION query depends on accumulated times 
+         * since the POSITION query depends on accumulated times and
+         * we cannot accumulate the current segment before the previous
+         * one completed.
          */
       default:
         return FALSE;
@@ -679,13 +793,16 @@ out_of_segment:
  * release the PREROLL_LOCK so that other threads can interrupt the entry.
  */
 static GstClockReturn
-gst_base_sink_wait_clock (GstBaseSink * basesink, GstClockTime time)
+gst_base_sink_wait_clock (GstBaseSink * basesink, GstClockTime time,
+    GstClockTimeDiff * jitter)
 {
   GstClockID id;
   GstClockReturn ret;
   GstClock *clock;
   GstClockTime base_time;
 
+  *jitter = 0;
+
   if (G_UNLIKELY (!GST_CLOCK_TIME_IS_VALID (time)))
     goto invalid_time;
 
@@ -704,7 +821,7 @@ gst_base_sink_wait_clock (GstBaseSink * basesink, GstClockTime time)
   /* release the preroll lock while waiting */
   GST_PAD_PREROLL_UNLOCK (basesink->sinkpad);
 
-  ret = gst_clock_id_wait (id, NULL);
+  ret = gst_clock_id_wait (id, jitter);
 
   GST_PAD_PREROLL_LOCK (basesink->sinkpad);
   gst_clock_id_unref (id);
@@ -748,9 +865,10 @@ no_clock:
  */
 static GstFlowReturn
 gst_base_sink_do_sync (GstBaseSink * basesink, GstPad * pad,
-    GstMiniObject * obj)
+    GstMiniObject * obj, gboolean * late)
 {
   GstClockTime start, stop;
+  GstClockTimeDiff jitter;
   gboolean syncable;
   GstClockReturn status = GST_CLOCK_OK;
 
@@ -795,7 +913,7 @@ again:
   /* preroll done, we can sync since we ar in PLAYING now. */
   GST_DEBUG_OBJECT (basesink, "waiting for clock");
   basesink->end_time = stop;
-  status = gst_base_sink_wait_clock (basesink, start);
+  status = gst_base_sink_wait_clock (basesink, start, &jitter);
   GST_DEBUG_OBJECT (basesink, "clock returned %d", status);
 
   /* waiting could be interrupted and we can be flushing now */
@@ -809,7 +927,25 @@ again:
     goto again;
   }
 
+  *late = FALSE;
+
   /* FIXME, update clock stats here and do some QoS */
+  if (status == GST_CLOCK_EARLY && GST_IS_BUFFER (obj)) {
+    if (basesink->abidata.ABI.max_lateness != -1
+        && jitter > basesink->abidata.ABI.max_lateness) {
+      GstEvent *event;
+
+      GST_DEBUG_OBJECT (basesink, "late: jitter!! %" G_GINT64_FORMAT "\n",
+          jitter);
+      *late = TRUE;
+
+      /* generate QoS event, FIXME, calculate decent proportion. */
+      event = gst_event_new_qos (-1.0, jitter, GST_BUFFER_TIMESTAMP (obj));
+
+      /* send upstream */
+      gst_pad_push_event (basesink->sinkpad, event);
+    }
+  }
 
   return GST_FLOW_OK;
 
@@ -843,14 +979,19 @@ gst_base_sink_render_object (GstBaseSink * basesink, GstPad * pad,
 {
   GstFlowReturn ret = GST_FLOW_OK;
   GstBaseSinkClass *bclass;
+  gboolean late = FALSE;
 
   /* synchronize this object */
-  ret = gst_base_sink_do_sync (basesink, pad, obj);
+  ret = gst_base_sink_do_sync (basesink, pad, obj, &late);
   if (G_UNLIKELY (ret != GST_FLOW_OK))
     goto sync_failed;
 
   /* and now render */
   if (G_LIKELY (GST_IS_BUFFER (obj))) {
+    /* drop late messages unconditionally */
+    if (late)
+      goto dropped;
+
     bclass = GST_BASE_SINK_GET_CLASS (basesink);
 
     GST_DEBUG_OBJECT (basesink, "rendering buffer %p", obj);
@@ -910,6 +1051,12 @@ sync_failed:
 
     return ret;
   }
+dropped:
+  {
+    GST_DEBUG_OBJECT (basesink, "buffer late, dropping, unref object %p", obj);
+    gst_mini_object_unref (obj);
+    return GST_FLOW_OK;
+  }
 }
 
 /* with STREAM_LOCK, PREROLL_LOCK
