@@ -33,18 +33,22 @@
  *   <listitem><para>live sources</para></listitem>
  * </itemizedlist>
  *
+ * <refsect2>
+ * <para>
  * The source can be configured to operate in a any #GstFormat with the
  * gst_base_src_set_format(). This format determines the format of the
  * internal #GstSegment and the #GST_EVENT_NEW_SEGMENT. The default format for
  * #GstBaseSrc is GST_FORMAT_BYTES.
- *
+ * </para>
+ * <para>
  * #GstBaseSrc always supports the push mode scheduling. If the following
  * conditions are met, it also supports pull mode scheduling:
  * <itemizedlist>
  *   <listitem><para>the format is set to GST_FORMAT_BYTES (default).</para></listitem>
  *   <listitem><para>#GstBaseSrc::is_seekable returns TRUE.</para></listitem>
  * </itemizedlist>
- *
+ * </para>
+ * <para>
  * If all the conditions are met for operating in pull mode, #GstBaseSrc is
  * automatically seekable in push mode as well. The following conditions must be
  * met to make the element seekable in push mode when the format is not
@@ -61,34 +65,41 @@
  *     #GstBaseSrc::do_seek is implemented, performs the seek and returns TRUE.
  *   </para></listitem>
  * </itemizedlist>
- *
+ * </para>
+ * <para>
  * When the default format is not GST_FORMAT_BYTES, the element should ignore the
  * offset and length in the #GstBaseSrc::create method. It is recommended to subclass
  * #GstPushSrc instead in this situation.
- *
+ * </para>
+ * <para>
  * #GstBaseSrc has support for live sources. Live sources are sources that produce
  * data at a fixed rate, such as audio or video capture. A typical live source also
  * provides a clock to export the rate at which they produce data.
  * Use gst_base_src_set_live() to activate the live source mode.
- *
+ * </para>
+ * <para>
  * A live source does not produce data in the PAUSED state, this means that the 
  * #GstBaseSrc::create method will not be called in PAUSED but only in PLAYING.
  * To signal the pipeline that the element will not produce data, its return
  * value from the READY to PAUSED state will be GST_STATE_CHANGE_NO_PREROLL.
- * 
+ * </para>
+ * <para>
  * A typical live source will timestamp the buffers they create with the current
  * stream time of the pipeline. This is why they can only produce data in PLAYING,
  * when the clock is actually distributed and running.
- *
+ * </para>
+ * <para>
  * The #GstBaseSrc::get_times method can be used to implement pseudo-live sources.
  * The base source will wait for the specified stream time returned in 
  * #GstBaseSrc::get_times before pushing out the buffer. It only makes sense to implement
  * the ::get_times function if the source is a live source.
- *
+ * </para>
+ * <para>
  * There is only support in GstBaseSrc for one source pad, which should be named
  * "src". A source implementation (subclass of GstBaseSrc) should install a pad
  * template in its base_init function, like so:
- *
+ * </para>
+ * <para>
  * <programlisting>
  * static void
  * my_element_base_init (gpointer g_class)
@@ -102,8 +113,46 @@
  *   gst_element_class_set_details (gstelement_class, &amp;details);
  * }
  * </programlisting>
- *
- * Last reviewed on 2005-12-18 (0.10.0)
+ * </para>
+ * <title>Controlled shutdown of live sources in applications</title>
+ * <para>
+ * Applications that record from a live source may want to stop recording
+ * in a controlled way, so that the recording is stopped, but the data
+ * already in the pipeline processed to the end (remember that many live
+ * sources would go on recording forever otherwise). For that to happen the
+ * application needs to make the source stop recording and send an EOS
+ * event down the pipeline. The application would then wait for an
+ * EOS message posted on the pipeline's bus to know when all data has
+ * been processed and the pipeline can safely be stopped.
+ * </para>
+ * <para>
+ * Since GStreamer 0.10.3 an application may simply set the source
+ * element to NULL or READY state to make it send an EOS event downstream.
+ * The application should lock the state of the source afterwards, so that
+ * shutting down the pipeline from PLAYING doesn't temporarily start up the
+ * source element for a second time:
+ * <programlisting>
+ * ...
+ * // stop recording
+ * gst_element_set_state (audio_source, GST_STATE_NULL);
+ * gst_element_set_locked_state (audio_source, TRUE);
+ * ...
+ * </programlisting>
+ * Now the application should wait for an EOS message
+ * to be posted on the pipeline's bus. Once it has received
+ * an EOS message, it may safely shut down the entire pipeline:
+ * <programlisting>
+ * ...
+ * // everything done - shut down pipeline
+ * gst_element_set_state (pipeline, GST_STATE_NULL);
+ * gst_element_set_locked_state (audio_source, FALSE);
+ * ...
+ * </programlisting>
+ * </para>
+ * <para>
+ * Last reviewed on 2006-03-07 (0.10.4)
+ * </para>
+ * </refsect2>
  */
 
 #include <stdlib.h>
@@ -151,6 +200,16 @@ enum
   PROP_TYPEFIND,
 };
 
+#define GST_BASE_SRC_GET_PRIVATE(obj)  \
+   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_BASE_SRC, GstBaseSrcPrivate))
+
+struct _GstBaseSrcPrivate
+{
+  gboolean last_sent_eos;       /* last thing we did was send an EOS (we set this
+                                 * to avoid the sending of two EOS in some cases) */
+  gboolean discont;
+};
+
 static GstElementClass *parent_class = NULL;
 
 static void gst_base_src_base_init (gpointer g_class);
@@ -210,7 +269,8 @@ static GstStateChangeReturn gst_base_src_change_state (GstElement * element,
     GstStateChange transition);
 
 static void gst_base_src_loop (GstPad * pad);
-static gboolean gst_base_src_check_get_range (GstPad * pad);
+static gboolean gst_base_src_pad_check_get_range (GstPad * pad);
+static gboolean gst_base_src_default_check_get_range (GstBaseSrc * bsrc);
 static GstFlowReturn gst_base_src_pad_get_range (GstPad * pad, guint64 offset,
     guint length, GstBuffer ** buf);
 static GstFlowReturn gst_base_src_get_range (GstBaseSrc * src, guint64 offset,
@@ -231,6 +291,8 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
+  g_type_class_add_private (klass, sizeof (GstBaseSrcPrivate));
+
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_base_src_finalize);
@@ -258,6 +320,8 @@ gst_base_src_class_init (GstBaseSrcClass * klass)
   klass->event = GST_DEBUG_FUNCPTR (gst_base_src_default_event);
   klass->do_seek = GST_DEBUG_FUNCPTR (gst_base_src_default_do_seek);
   klass->query = GST_DEBUG_FUNCPTR (gst_base_src_default_query);
+  klass->check_get_range =
+      GST_DEBUG_FUNCPTR (gst_base_src_default_check_get_range);
 }
 
 static void
@@ -266,6 +330,8 @@ gst_base_src_init (GstBaseSrc * basesrc, gpointer g_class)
   GstPad *pad;
   GstPadTemplate *pad_template;
 
+  basesrc->priv = GST_BASE_SRC_GET_PRIVATE (basesrc);
+
   basesrc->is_live = FALSE;
   basesrc->live_lock = g_mutex_new ();
   basesrc->live_cond = g_cond_new ();
@@ -291,7 +357,7 @@ gst_base_src_init (GstBaseSrc * basesrc, gpointer g_class)
       GST_DEBUG_FUNCPTR (gst_base_src_event_handler));
   gst_pad_set_query_function (pad, GST_DEBUG_FUNCPTR (gst_base_src_query));
   gst_pad_set_checkgetrange_function (pad,
-      GST_DEBUG_FUNCPTR (gst_base_src_check_get_range));
+      GST_DEBUG_FUNCPTR (gst_base_src_pad_check_get_range));
   gst_pad_set_getrange_function (pad,
       GST_DEBUG_FUNCPTR (gst_base_src_pad_get_range));
   gst_pad_set_getcaps_function (pad, GST_DEBUG_FUNCPTR (gst_base_src_getcaps));
@@ -785,6 +851,7 @@ gst_base_src_perform_seek (GstBaseSrc * src, GstEvent * event, gboolean unlock)
             src->segment.last_stop, stop, src->segment.time));
   }
 
+  src->priv->discont = TRUE;
   src->data.ABI.running = TRUE;
   /* and restart the task in case it got paused explicitely or by
    * the FLUSH_START event we pushed out. */
@@ -1123,13 +1190,14 @@ gst_base_src_get_range (GstBaseSrc * src, guint64 offset, guint length,
       src->num_buffers_left--;
   }
 
+  GST_DEBUG_OBJECT (src,
+      "calling create offset %" G_GUINT64_FORMAT " %" G_GINT64_FORMAT, offset,
+      src->segment.time);
+
   ret = bclass->create (src, offset, length, buf);
   if (ret != GST_FLOW_OK)
     goto done;
 
-  GST_DEBUG_OBJECT (src, "offset %" G_GUINT64_FORMAT " %" G_GINT64_FORMAT,
-      offset, src->segment.time);
-
   /* no timestamp set and we are at offset 0 */
   if (offset == 0 && src->segment.time == 0
       && GST_BUFFER_TIMESTAMP (*buf) == -1)
@@ -1202,14 +1270,37 @@ gst_base_src_pad_get_range (GstPad * pad, guint64 offset, guint length,
 }
 
 static gboolean
-gst_base_src_check_get_range (GstPad * pad)
+gst_base_src_pad_check_get_range (GstPad * pad)
 {
+  GstBaseSrcClass *bclass;
   GstBaseSrc *src;
   gboolean res;
 
   src = GST_BASE_SRC (gst_pad_get_parent (pad));
 
+  bclass = GST_BASE_SRC_GET_CLASS (src);
+
+  if (bclass->check_get_range == NULL) {
+    GST_WARNING_OBJECT (src, "no check_get_range function set");
+    return FALSE;
+  }
+
+  res = bclass->check_get_range (src);
+  GST_LOG_OBJECT (src, "%s() returned %d",
+      GST_DEBUG_FUNCPTR_NAME (bclass->check_get_range), (gint) res);
+
+  gst_object_unref (src);
+
+  return res;
+}
+
+static gboolean
+gst_base_src_default_check_get_range (GstBaseSrc * src)
+{
+  gboolean res;
+
   if (!GST_OBJECT_FLAG_IS_SET (src, GST_BASE_SRC_STARTED)) {
+    GST_LOG_OBJECT (src, "doing start/stop to check get_range support");
     gst_base_src_start (src);
     gst_base_src_stop (src);
   }
@@ -1219,8 +1310,6 @@ gst_base_src_check_get_range (GstPad * pad)
    * flag and is set in the _start() method. */
   res = src->random_access;
 
-  gst_object_unref (src);
-
   return res;
 }
 
@@ -1234,6 +1323,8 @@ gst_base_src_loop (GstPad * pad)
 
   src = GST_BASE_SRC (gst_pad_get_parent (pad));
 
+  src->priv->last_sent_eos = FALSE;
+
   /* if we operate in bytes, we can calculate an offset */
   if (src->segment.format == GST_FORMAT_BYTES)
     position = src->segment.last_stop;
@@ -1281,6 +1372,12 @@ gst_base_src_loop (GstPad * pad)
   if (position != -1)
     gst_segment_set_last_stop (&src->segment, src->segment.format, position);
 
+  if (G_UNLIKELY (src->priv->discont)) {
+    buf = gst_buffer_make_metadata_writable (buf);
+    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+    src->priv->discont = FALSE;
+  }
+
   ret = gst_pad_push (pad, buf);
   if (G_UNLIKELY (ret != GST_FLOW_OK))
     goto pause;
@@ -1302,6 +1399,7 @@ eos:
               src->segment.format, src->segment.last_stop));
     } else {
       gst_pad_push_event (pad, gst_event_new_eos ());
+      src->priv->last_sent_eos = TRUE;
     }
     goto done;
   }
@@ -1317,6 +1415,7 @@ pause:
           (_("Internal data flow error.")),
           ("streaming task paused, reason %s", reason));
       gst_pad_push_event (pad, gst_event_new_eos ());
+      src->priv->last_sent_eos = TRUE;
     }
     goto done;
   }
@@ -1328,6 +1427,7 @@ error:
     src->data.ABI.running = FALSE;
     gst_pad_pause_task (pad);
     gst_pad_push_event (pad, gst_event_new_eos ());
+    src->priv->last_sent_eos = TRUE;
     goto done;
   }
 }
@@ -1598,6 +1698,8 @@ gst_base_src_activate_push (GstPad * pad, gboolean active)
     if (!gst_base_src_start (basesrc))
       goto error_start;
 
+    basesrc->priv->last_sent_eos = FALSE;
+
     /* do initial seek, which will start the task */
     GST_OBJECT_LOCK (basesrc);
     event = basesrc->data.ABI.pending_seek;
@@ -1652,6 +1754,9 @@ gst_base_src_activate_pull (GstPad * pad, gboolean active)
   } else {
     GST_DEBUG_OBJECT (basesrc, "Deactivating in pull mode");
 
+    /* don't send EOS when going from PAUSED => READY when in pull mode */
+    basesrc->priv->last_sent_eos = TRUE;
+
     if (!gst_base_src_stop (basesrc))
       goto error_stop;
 
@@ -1689,6 +1794,8 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
         no_preroll = TRUE;
         basesrc->live_running = FALSE;
       }
+      basesrc->priv->last_sent_eos = FALSE;
+      basesrc->priv->discont = TRUE;
       GST_LIVE_UNLOCK (element);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
@@ -1720,7 +1827,12 @@ gst_base_src_change_state (GstElement * element, GstStateChange transition)
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       if (!gst_base_src_stop (basesrc))
         goto error_stop;
-      gst_pad_push_event (basesrc->srcpad, gst_event_new_eos ());
+
+      if (!basesrc->priv->last_sent_eos) {
+        GST_DEBUG_OBJECT (basesrc, "Sending EOS event");
+        gst_pad_push_event (basesrc->srcpad, gst_event_new_eos ());
+        basesrc->priv->last_sent_eos = TRUE;
+      }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
